grammar Fexpression;

tokens {
    AND = '&&';
    OR = '||';
    NOT = '!';
    
    TRUE = 'true';
    FALSE = 'false';
    
    LPAREN = '(';
    RPAREN = ')';
}

@lexer::header{
    package be.unamur.fts.fexpression;
}

@parser::header{
    package be.unamur.fts.fexpression;
}

@parser::members{
	private FExpression root = null;
	
	public FExpression getRoot(){
		return this.root;
	}
}


epxression: 
	expr=boolExpression							{ this.root = $expr.node; }
;


boolExpression returns [FExpression node] options { backtrack=true; } :
	left=boolExpressionPrior1 					{ $node = $left.node; }
		(	AND andRight=boolExpressionPrior1	{ $node = new AndNode($node,$andRight.node); }
		|	OR  orRight=boolExpressionPrior1	{ $node = new OrNode($node,$orRight.node); }
		)*
;

boolExpressionPrior1 returns [FExpression node]:  
	  LPAREN expr=boolExpression RPAREN 		{ $node = $expr.node; }
	| TRUE										{ $node = ConstantNode.TRUE; }
	| FALSE										{ $node = ConstantNode.FALSE; }
	| FEATURE									{ $node = new FeatureNode($FEATURE.text); }
	| NOT negExpr=boolExpressionPrior1			{ $node = new NotNode($negExpr.node); }
;




FEATURE: 
	LETTER (LETTER | DIGIT)* 
;

// Whitespaces -> ignored

NL: 
	'\n' | '\r'
;
 
WS: 
	(' ' | '\t' | NL) { skip(); }
;

fragment LETTER
	:	'A'..'Z' | 'a'..'z' | '_';

fragment DIGIT
	:	'0'..'9';